﻿using System;
using Ozeki.VoIP;
using System.Timers;
using Ozeki.Media;

namespace _04_Multi_Level_IVR
{
    class CallHandler
    {
        ICall _call;
        MediaConnector _mediaConnector;
        PhoneCallAudioSender _phoneCallAudioSender;
        Timer _greetingMessageTimer;
        string _blindTransferNumber;
        TextToSpeech _tts;
        MP3StreamPlayback _mp3Player;

        public event EventHandler Completed;

        public CallHandler(ICall call)
        {
            _greetingMessageTimer = new Timer();
            _greetingMessageTimer.Interval = 20000;
            _greetingMessageTimer.Elapsed += greetingMessageTimer_Elapsed;
            _call = call;
            _phoneCallAudioSender = new PhoneCallAudioSender();
            _phoneCallAudioSender.AttachToCall(call);
            _mediaConnector = new MediaConnector();
        }

        public void Start()
        {
            _call.CallStateChanged += call_CallStateChanged;
            _call.DtmfReceived += call_DtmfReceived;
            _call.Answer();
        }

        void call_DtmfReceived(object sender, VoIPEventArgs<DtmfInfo> e)
        {
            DisposeCurrentConnector();
            switch (e.Item.Signal.Signal)
            {
                case 0: break;
                case 1: TextToSpeech("Ozeki Informatics Ltd. is a leading mobile messaging software vendor and Ozeki VoIP SIP SDK is an excellent software development kit that allows you to establish VoIP calls from your application easily and quickly. You do not need to have expert programming skills, with the most basic programming knowledge you will be able to create extraordinary VoIP solutions with this tool."); break;
                case 2: MP3ToSpeaker(); break;
                case 3: LowerMenu(); break;
            }
        }

        private void LowerMenu()
        {
            _call.CallStateChanged -= call_CallStateChanged;
            _call.DtmfReceived -= call_DtmfReceived;
            _greetingMessageTimer.Elapsed -= greetingMessageTimer_Elapsed;

            NewCallHandler newCallHandler = new NewCallHandler(_call);
            newCallHandler.BlindTransferNumber(_blindTransferNumber);
            newCallHandler.Completed += newCallHandler_Completed;
            newCallHandler.Start();
        }

        void newCallHandler_Completed(object sender, EventArgs e)
        {
            _call.CallStateChanged +=call_CallStateChanged;
            _call.DtmfReceived +=call_DtmfReceived;
            StartGreetingMessage();
            _greetingMessageTimer.Start();
        }

        public void BlindTransferNumber(string blindTransfer)
        {
            _blindTransferNumber = blindTransfer;
        }

        void greetingMessageTimer_Elapsed(object sender, ElapsedEventArgs e)
        {
            StartGreetingMessage();
        }

        private void StartGreetingMessage()
        {
            DisposeCurrentConnector();
            TextToSpeech("Introduce you the Interactive Voice Mail aka IVR example code written with Ozeki VoIP SDK. For more information about Ozeki Ltd. please press button one. By pressing button two, you can listen a sample mp3 file. If you press button three, you can reach a lower menu level.");
        }

        private void MP3ToSpeaker()
        {
            _greetingMessageTimer.Stop();

            _mp3Player = new MP3StreamPlayback("../../test.mp3");
            
            _mediaConnector.Connect(_mp3Player, _phoneCallAudioSender);
            _mp3Player.Stopped += mp3Player_Stopped;
            _mp3Player.Start();
        }

        void call_CallStateChanged(object sender, CallStateChangedArgs e)
        {
            if (e.State == CallState.Answered)
            {
                Console.WriteLine("Call answered!");
                _greetingMessageTimer.Start();
                StartGreetingMessage();
            }
        }

        private void TextToSpeech(string text)
        {
            _greetingMessageTimer.Stop();

            _tts = new TextToSpeech();
            _tts.Stopped += tts_Stopped;

            _mediaConnector.Connect(_tts, _phoneCallAudioSender);
            _tts.AddAndStartText(text);
        }

        void mp3Player_Stopped(object sender, EventArgs e)
        {
            _greetingMessageTimer.Start();
        }

        void tts_Stopped(object sender, EventArgs e)
        {
            _greetingMessageTimer.Start();
        }

        private void DisposeCurrentConnector()
        {
            if (_tts != null)
            {
                _mediaConnector.Disconnect(_tts, _phoneCallAudioSender);
            }
            if (_mp3Player != null)
            {
                _mediaConnector.Disconnect(_mp3Player, _phoneCallAudioSender);
            }
        }
    }
}
