﻿using System;
using Ozeki.VoIP;
using System.Timers;
using Ozeki.Media;

namespace _04_Multi_Level_IVR
{
    class NewCallHandler
    {
        ICall _call;
        PhoneCallAudioSender _phoneCallAudioSender;
        MediaConnector _mediaConnector;
        public event EventHandler Completed;
        string _blindTransferNumber;
        Timer _greetingMessageTimer;
        TextToSpeech _tts;

        public NewCallHandler(ICall call)
        {
            _call = call;
            _phoneCallAudioSender = new PhoneCallAudioSender();
            _phoneCallAudioSender.AttachToCall(call);
            _mediaConnector = new MediaConnector();
            _greetingMessageTimer = new Timer();
            _greetingMessageTimer.Elapsed += greetingMessageTimer_Elapsed;
            _greetingMessageTimer.Interval = 15000;
        }

        public void Start()
        {
            _call.CallStateChanged += call_CallStateChanged;
            _call.DtmfReceived += call_DtmfReceived;
            _greetingMessageTimer.Start();
            StartGreetingMessage();
        }

        private void StartGreetingMessage()
        {
            DisposeCurrentConnector();
            TextToSpeech("You entered a lower menu. If you would like to transfer to the added number, please press button 1. By pressing button 2, you are returning to the main menu.");
        }

        private void TextToSpeech(string p)
        {
            _greetingMessageTimer.Stop();
            _tts = new TextToSpeech();
            _tts.Stopped += tts_Stopped;
            _mediaConnector.Connect(_tts, _phoneCallAudioSender);
            _tts.AddAndStartText(p);
        }

        void tts_Stopped(object sender, EventArgs e)
        {
            _greetingMessageTimer.Start();
        }

        private void DisposeCurrentConnector()
        {
            if (_tts != null)
            {
                _mediaConnector.Disconnect(_tts, _phoneCallAudioSender);
            }
        }

        void call_DtmfReceived(object sender, VoIPEventArgs<DtmfInfo> e)
        {
            DisposeCurrentConnector();
            switch (e.Item.Signal.Signal)
            {
                case 1:
                    {
                        if (_blindTransferNumber == "0")
                        {
                            TextToSpeech("You did not add any number for blind transferring!");
                            break;
                        }
                        else
                        {
                            _call.BlindTransfer(_blindTransferNumber);
                            break;
                        }
                    }
                case 2:
                    {
                        Return(); break;
                    }
            }
        }

        private void Return()
        {
            _call.DtmfReceived -= call_DtmfReceived;
            DisposeCurrentConnector();
            _greetingMessageTimer.Close();

            var handler = Completed;
            if (handler != null)
                handler(this, EventArgs.Empty);
        }

        public void BlindTransferNumber(string blindTransferNumber)
        {
            _blindTransferNumber = blindTransferNumber;
        }

            void call_CallStateChanged(object sender, CallStateChangedArgs e)
            {
                if (e.State == CallState.InCall) _greetingMessageTimer.Start();
                if (e.State == CallState.Transferring) DisposeCurrentConnector(); _greetingMessageTimer.Stop();
            }

        void greetingMessageTimer_Elapsed(object sender, ElapsedEventArgs e)
        {
            StartGreetingMessage();
        }
    }
}
